
from torch import nn
import torchvision
import torch
import torch.nn.functional as F
import warnings
warnings.filterwarnings("ignore", category=UserWarning)


class bscp(nn.Module):

    def __init__(self, channels_1, channels_2, rank, l1, l2, p):
        super(bscp, self).__init__()

        self.channels_1 = channels_1
        self.channels_2 = channels_2
        self.rank = rank
        self.l1 = l1
        self.l2 = l2
        self.p = p

        # l1,l2
        self.linear_1 = nn.Linear(self.channels_2 * self.rank, self.l2)
        self.linear_2 = nn.Linear(self.channels_1 * self.rank, self.l1)

        nn.init.xavier_normal_(self.linear_1.weight.data)
        self.linear_1.bias.data.zero_()
        nn.init.xavier_normal_(self.linear_2.weight.data)
        self.linear_2.bias.data.zero_()


        self.h = nn.Linear(self.l1 * self.l2, self.p)
        nn.init.xavier_normal_(self.h.weight.data)
        self.h.bias.data.zero_()

    def forward(self, x, y):

        x = torch.einsum('imjk,injk->imn', x, y)  # [b,m,n]
        x = x.repeat(1, self.rank, self.rank)
        x = self.linear_1(x)
        x = x.permute(0, 2, 1)
        x = self.linear_2(x)
        x1 = x.view(-1, self.l1 * self.l2)
        x1 = self.h(x1)

        x1 = F.normalize(x1, p=2, dim=1)

        return x1


class BSCP_Resnet50(nn.Module):

    def __init__(self, ch_1, ch_2, r, l1, l2, p, num_classes):
        super(BSCP_Resnet50, self).__init__()

        self.features = torchvision.models.resnet50(pretrained=True)
        self.features = nn.Sequential(*list(self.features.children())[:-2])

        self.bscp = bscp(channels_1=ch_1, channels_2=ch_2, rank=r, l1=l1, l2=l2, p=p)

        self.fc = nn.Linear(p, num_classes)
        nn.init.xavier_normal_(self.fc.weight.data)
        self.fc.bias.data.zero_()

    def forward(self, x):

        # [batch, 2048, 14, 14]
        feature_1 = self.features(x)
        feature_1 = self.bscp(feature_1, feature_1)
        feature_1 = self.fc(feature_1)

        return feature_1


if __name__ == '__main__':

    input = torch.randn(6, 3, 448, 448)
    model = BSCP_Resnet50(512, 512, 2, 250, 250, 512, 200)
    result = model(input)
    print(result.shape)
    print(sum(x.numel() for x in model.parameters()))
