#!/usr/bin/python3.6
# -*- coding: utf-8 -*-
# Copyright (C) 2022 #
# @Time    : 2022/7/20 上午11:15
# @Author  : Lihao  Anhui University
# @Email   : lihao6897@gmail.com
# @File    : CUB2.py
# @Software: PyCharm
from torch import nn
import torchvision
from config import opt
import torch
import torch.nn.functional as F


class BSCP(nn.Module):

    def __init__(self, channels_1, channels_2, rank, l1, l2, p):
        super(BSCP, self).__init__()

        self.channels_1 = channels_1
        self.channels_2 = channels_2
        self.rank = rank
        self.l1 = l1
        self.l2 = l2
        self.p = p

        # l1,l2
        self.linear_1 = nn.Linear(self.channels_2 * self.rank, self.l2)
        self.linear_2 = nn.Linear(self.channels_1 * self.rank, self.l1)

        nn.init.xavier_normal_(self.linear_1.weight.data)
        self.linear_1.bias.data.zero_()
        nn.init.xavier_normal_(self.linear_2.weight.data)
        self.linear_2.bias.data.zero_()

        self.h = nn.Linear(self.l1 * self.l2, self.p)
        nn.init.xavier_normal_(self.h.weight.data)
        self.h.bias.data.zero_()

    def forward(self, x, y):

        x = torch.einsum('imjk,injk->imn', x, y)  # [b,m,n]
        x = x.repeat(1, self.rank, self.rank)
        x = self.linear_1(x)
        x = x.permute(0, 2, 1)
        x = self.linear_2(x)
        x = x.view(-1, self.l1 * self.l2)
        x = self.h(x)
        x = x.unsqueeze(dim=2).unsqueeze(dim=3)

        return x


class BSCP_kernelPooling(nn.Module):

    def __init__(self, rank=[2, 2, 2], l1=[512, 512, 512], l2=[512, 512, 512], p=[512, 512, 512], num_classes=200, use=2):
        super(BSCP_kernelPooling, self).__init__()

        self.use = use

        self.features = torchvision.models.resnet50(pretrained=True)
        self.features = nn.Sequential(*list(self.features.children())[:-1])
        self.bscp_2 = BSCP(channels_1=2048, channels_2=2048, rank=rank[0], l1=l1[0], l2=l2[0], p=p[0])
        self.bscp_3 = BSCP(channels_1=2048, channels_2=p[0], rank=rank[1], l1=l1[1], l2=l2[1], p=p[1])

        if use == 2:
            total = p[0] + p[1]
        else:
            self.bscp_4 = BSCP(channels_1=2048, channels_2=p[1], rank=rank[2], l1=l1[2], l2=l2[2], p=p[2])
            total = p[0] + p[1] + p[2]
        self.fc = nn.Linear(total, num_classes)            # todo  goal       相当于embedding？
        nn.init.xavier_normal_(self.fc.weight.data)
        self.fc.bias.data.zero_()

    def forward(self, x):

        # [batch, 3, 448, 448] ->  1-order
        feature_1 = self.features(x)
        feature_2 = self.bscp_2(feature_1, feature_1)       # 2-order
        feature_3 = self.bscp_3(feature_1, feature_2)       # 3-order

        # output = torch.cat([feature_1, feature_2], dim=1)
        output = torch.cat([feature_2, feature_3], dim=1)      # [batch, 3072, 1, 1]    [todo, feature_3]
        if self.use == 3:
            feature_4 = self.bscp_4(feature_1, feature_3)  # 4-order
            output = torch.cat([output, feature_4], dim=1)      # [batch, 3584, 1, 1]    [todo, feature_3]
        output = torch.flatten(output, 1)              # [batch, 3072]
        output = F.normalize(output, p=2, dim=1)
        output = self.fc(output)

        return output


if __name__ == '__main__':
    input = torch.randn(2, 3, 448, 448)
    model = BSCP_kernelPooling(use=3)
    result = model(input)
    print(model)
    print(result.shape)
    print(sum(x.numel() for x in model.parameters()))
