
import matplotlib.pyplot as plt
import numpy as np
import torch
import cv2
from PIL import Image


def plot_kernel_feature(model, writer=None):
    """
    当前这个模型只需要输出卷积核的特征图，可以根据自己的需要进行修改
    :param model:
    :param writer:
    :param layer:
    :return:
    """
    for name, para in model.named_parameters():
        print(name)
        # if 'conv' in name and 'weight' in name:
        if 'features' in name and 'weight' in name:
            if writer is not None:
                for i in range(para.shape[1]):
                    writer.add_image(name+f'/{i}', para[0][i].detach().cpu().unsqueeze(dim=0), 0)
            else:
                if len(para.size()) != 4:
                    continue
                print(para.shape)
                plt.figure(figsize=(12, 12))
                sum_para = para.sum(dim=1)
                for i in range(min(sum_para.shape[0], 64)):
                    plt.subplot(8, 8, i+1)
                    plt.imshow(sum_para[i, :, :].cpu().detach().numpy())
                    plt.axis('off')
                plt.show()

                # plt.figure(figsize=(12, 12))
                # for i in range(para.shape[1]):
                #     plt.subplot(8, 8, i+1)
                #     plt.imshow(para[0, i, :, :].cpu().detach().numpy())
                #     # plt.imshow(para[i, :, :, :].detach().numpy())
                #     plt.axis('off')
                # plt.show()


def plot_feature_map(args, model, test_loader, map):

    dataiter = iter(test_loader)
    images, labels = dataiter.__next__()
    img = images[0].unsqueeze(dim=0).to(args.device)

    _ = model(img)

    img = img[0].cpu().numpy().transpose(1, 2, 0)
    img = (img * [0.1817, 0.1811, 0.1927] + [0.4856, 0.4994, 0.4324]) * 255
    plt.imshow(img.astype('uint8'))
    plt.show()

    feature = map['conv2d'].cpu()
    print(feature.shape)
    plt.figure(figsize=(12, 12))
    for i in range(min(feature.shape[1], 64)):
        plt.subplot(8, 8, i + 1)
        plt.imshow(feature[0, i, :, :])
        plt.axis('off')
    plt.show()


def plt_attention_map(args, model, test_loader, map, type='qkv'):
    assert type in ['qkv', 'original']

    dataiter = iter(test_loader)
    images, labels = dataiter.__next__()
    b, c, w, h = images.size()
    images = images.to(args.device)
    _ = model(images)

    feature = map['conv2d'].cpu().detach().numpy()
    print(feature.shape)

    plt.figure(figsize=(12, 12))
    for i in range(min(feature.shape[0], 6)):
        plt.subplot(2, 3, i + 1, xticks=[], yticks=[])
        img = images[i].cpu().numpy().transpose(1, 2, 0)
        img = (img * [0.1817, 0.1811, 0.1927] + [0.4856, 0.4994, 0.4324]) * 255
        plt.imshow(img.astype('uint8'), alpha=1)
        if type == 'qkv':
            # att = feature[i].sum(axis=0) / 196
            # att = att.reshape(14, 14)
            att = feature[i].sum(axis=0) / 12544
            att = att.reshape(112, 112)
        else:
            att = feature[i][0]

        att = cv2.resize(att, (w, h))
        att = att / att.max()
        att = (att * 255).astype('uint8')
        plt.imshow(att, alpha=0.5, interpolation='nearest', cmap='jet')

    plt.show()
