
import torch
import matplotlib.pyplot as plt
""""""


def create_scheduler(args, optimizer):

    schedu = args.scheduler.lower()

    if schedu == 'steplr':
        scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=args.step, gamma=args.gamma, last_epoch=-1)
    elif schedu == 'onecyclelr':
        scheduler = torch.optim.lr_scheduler.OneCycleLR(optimizer, total_steps=100, max_lr=args.max_lr, three_phase=True)
    elif schedu == 'multisteplr':
        scheduler = torch.optim.lr_scheduler.MultiStepLR(optimizer, milestones=args.milestones, gamma=args.gamma, last_epoch=-1)
    elif schedu == 'custom':             # todo
        scheduler = None
    else:
        scheduler = None
        assert False and "Invalid scheduler"

    return scheduler


if __name__ == '__main__':
    model = torch.nn.Linear(3, 32)

    optimizer = torch.optim.SGD(model.parameters(), lr=0.01, momentum=0.9, weight_decay=5e-4)
    scheduler = torch.optim.lr_scheduler.MultiStepLR(optimizer, [30], gamma=0.1, last_epoch=-1)
    print(scheduler.state_dict())

    lr_list = []
    for epoch in range(40):
        scheduler.step()
        lr_list.append(optimizer.state_dict()['param_groups'][0]['lr'])
    plt.plot(range(40), lr_list, color='r')
    plt.xlabel('epoch')
    plt.ylabel('lr')
    plt.title('step2')
    plt.show()
