
import torch
import torchvision
from torchvision import transforms
from torch.utils.data import RandomSampler


def load_path(datatype):
    if datatype == 'CUB':
        path = '../CUB_200_2011/'               # todo
        ROOT_TRAIN = path + 'dataset/train/'
        ROOT_TEST = path + 'dataset/test/'

    elif datatype == 'Cars':
        path = '../car196/'
        ROOT_TRAIN = path + 'car_train/'
        ROOT_TEST = path + 'car_test/'

    elif datatype == 'Indoor':
        path = '../indoorCVPR_09/'
        ROOT_TRAIN = path + 'dataset/train/'
        ROOT_TEST = path + 'dataset/test/'
    else:
        print("datatype error!!!")

    return ROOT_TRAIN, ROOT_TEST


def train_data_load(datatype, batch_size):
    data_transform = transforms.Compose([
        transforms.Resize((448, 448)),
        transforms.RandomHorizontalFlip(),
        transforms.RandomRotation(degrees=(-30, +30)),
        transforms.ToTensor(),
        transforms.Normalize(mean=[0.4856, 0.4994, 0.4324], std=[0.1817, 0.1811, 0.1927])
    ])
    root_train, _ = load_path(datatype)
    train_dataset = torchvision.datasets.ImageFolder(root_train,
                                                     transform=data_transform)
    CLASS = train_dataset.class_to_idx
    # elm_prop = {str(value): key for key, value in CLASS.items()}  # 把key由字符变为int example:  '1'->1
    # print(elm_prop)
    train_loader = torch.utils.data.DataLoader(train_dataset,
                                               batch_size=batch_size,
                                               shuffle=True)
    return train_loader


def test_data_load(datatype, batch_size):
    data_transform = transforms.Compose([
        transforms.Resize((448, 448)),
        transforms.CenterCrop(448),
        transforms.ToTensor(),
        transforms.Normalize(mean=[0.4856, 0.4994, 0.4324], std=[0.1817, 0.1811, 0.1927])
    ])
    _,root_test = load_path(datatype)
    test_dataset = torchvision.datasets.ImageFolder(root_test,
                                                    transform=data_transform)

    CLASS = test_dataset.class_to_idx
    test_loader = torch.utils.data.DataLoader(test_dataset,
                                              batch_size=batch_size,
                                              shuffle=False)
    return test_loader


def sample_test_data_load(datatype, batch_size):
    data_transform = transforms.Compose([
        transforms.Resize((448, 448)),
        transforms.CenterCrop(448),
        transforms.ToTensor(),
        transforms.Normalize(mean=[0.4856, 0.4994, 0.4324], std=[0.1817, 0.1811, 0.1927])
    ])
    _,root_test = load_path(datatype)
    sample_test_dataset = torchvision.datasets.ImageFolder(root_test,
                                                    transform=data_transform)

    sample = RandomSampler(sample_test_dataset)

    CLASS = sample_test_dataset.class_to_idx
    sample_test_loader = torch.utils.data.DataLoader(sample_test_dataset,
                                              batch_size=batch_size,
                                              shuffle=False,
                                              sampler=sample)
    return sample_test_loader


if __name__ == '__main__':
    train_loader = train_data_load('CUB', 32)
    # test_loader = test_data_load('CUB')
    # for i,(input,label) in enumerate(test_loader):
    #     print(input.shape)
    #     print(label.shape)
    #     input = input.permute(0,2,3,1)
    #     print(label)

