
import torch
from torch import nn
import torchvision
import torch.nn.functional as F
import warnings
warnings.filterwarnings("ignore", category=UserWarning)


# class RK(nn.Module):
#     def __init__(self, in_channel, out_channel):
#         super(RK, self).__init__()
#         self.in_channel = in_channel
#         self.fc1 = nn.Linear(in_channel, out_channel)
#         self.fc2 = nn.Linear(in_channel, out_channel)
#         nn.init.xavier_normal_(self.fc1.weight.data)
#         self.fc1.bias.data.zero_()
#         nn.init.xavier_normal_(self.fc2.weight.data)
#         self.fc2.bias.data.zero_()
#
#     def forward(self, x):
#         x = self.fc1(x).permute(0, 2, 1)
#         x = self.fc2(x)
#         return x


class LightBSCPResnet50(nn.Module):
    def __init__(self, in_channel=2048, out_channel=512, out_dim=512, rank=2, num_classes=200):
        super(LightBSCPResnet50, self).__init__()
        self.out_channel = out_channel

        self.features = torchvision.models.resnet50(pretrained=True)
        self.features = nn.Sequential(*list(self.features.children())[:-2])

        # l1,l2
        # self.layers = nn.ModuleList([RK(in_channel=in_channel, out_channel=out_channel) for _ in range(rank)])

        # l1,l2
        self.linear_1 = nn.Linear(in_channel, self.out_channel)
        self.linear_2 = nn.Linear(in_channel, self.out_channel)

        nn.init.xavier_normal_(self.linear_1.weight.data)
        self.linear_1.bias.data.zero_()
        nn.init.xavier_normal_(self.linear_2.weight.data)
        self.linear_2.bias.data.zero_()

        self.h = nn.Linear(out_channel * out_channel, out_dim)
        nn.init.xavier_normal_(self.h.weight.data)
        self.h.bias.data.zero_()
        self.fc = nn.Linear(out_dim, num_classes)
        nn.init.xavier_normal_(self.fc.weight.data)
        self.fc.bias.data.zero_()

    def forward(self, x):

        x = self.features(x)
        b, c, w, h = x.size()

        x = x.view(b, c, w*h).sum(dim=2)
        x = x.unsqueeze(dim=2)
        x_T = x.permute(0, 2, 1)
        x = torch.matmul(x, x_T)

        x = self.linear_1(x).permute(0, 2, 1)
        x = self.linear_2(x)
        x = x.view(-1, self.out_channel * self.out_channel)
        x = self.h(x)
        x = F.normalize(x, p=2, dim=1)
        x = self.fc(x)

        return x


if __name__ == '__main__':

    input = torch.randn(6, 3, 448, 448)
    model = LightBSCPResnet50(2048, 512, 512, 2, 200)
    result = model(input)
    print(result.shape)
